package cc.fascinated.services;

import cc.fascinated.platform.Platform;
import cc.fascinated.platform.impl.ScoreSaberPlatform;
import com.mongodb.client.model.Filters;
import lombok.NonNull;
import lombok.extern.log4j.Log4j2;
import org.bson.Document;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.context.annotation.DependsOn;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * @author Fascinated (fascinated7)
 */
@Service
@DependsOn("mongoService")
@Log4j2(topic = "PlatformService")
public class PlatformService {
    public static PlatformService INSTANCE;
    private static final ExecutorService EXECUTOR_SERVICE = Executors.newFixedThreadPool(1);

    /**
     * The loaded platforms.
     */
    private final List<Platform> platforms = new ArrayList<>();

    @Autowired
    public PlatformService(@NonNull ApplicationContext context) {
        INSTANCE = this;
        log.info("Registering platforms...");
        registerPlatform(context.getBean(ScoreSaberPlatform.class));
        log.info("Loaded %s platforms.".formatted(this.platforms.size()));
    }

    /**
     * Updates the platform metrics.
     * <p>
     * This method is scheduled to run every 5 minutes.
     * </p>
     */
    @Scheduled(cron = "0 */5 * * * *")
    public void updateMetrics() {
        log.info("Updating %s platform metrics...".formatted(this.platforms.size()));
        for (Platform platform : this.platforms) {
            platform.updateMetrics();
        }
        log.info("Finished updating platform metrics.");
    }

    /**
     * Updates the platform players.
     * <p>
     * This method is scheduled to run every hour.
     * </p>
     */
    @Scheduled(cron = "0 0 * * * *")
    public void updatePlayerMetrics() {
        log.info("Updating %s platform player metrics...".formatted(this.platforms.size()));
        for (Platform platform : this.platforms) {
            platform.trackPlayerMetrics();
        }
        log.info("Finished updating platform player metrics.");
    }

    /**
     * Refreshes the platform leaderboards.
     * <p>
     * This method is scheduled to run every day at midnight.
     * This is to ensure that the leaderboards are up-to-date.
     * </p>
     */
    @Scheduled(cron = "0 0 0 * * *")
    public void refreshPlatformLeaderboards() {
        log.info("Refreshing platform leaderboards...");
        EXECUTOR_SERVICE.execute(() -> {
            for (Platform platform : this.platforms) {
                platform.updateLeaderboards();
            }
        });
        log.info("Finished refreshing platform leaderboards.");
    }

    /**
     * Registers a platform.
     *
     * @param platform the platform to register
     */
    public void registerPlatform(Platform platform) {
        this.platforms.add(platform);
        log.info(" - Registered platform '%s'".formatted(platform.getPlatform().getPlatformName()));

        // Find the platform in the database
        Document document = MongoService.INSTANCE.getPlatformsCollection().find(Filters.eq("_id", platform.getPlatform().getPlatformName())).first();
        if (document == null) { // The platform was not found
            document = new Document("_id", platform.getPlatform().getPlatformName());
            MongoService.INSTANCE.getPlatformsCollection().insertOne(document);
        }

        // Set the previous curve version
        platform.setPreviousCurveVersion(document.getInteger("currentCurveVersion", platform.getCurrentCurveVersion()));

        // The curve was updated
        if (platform.getPreviousCurveVersion() != platform.getCurrentCurveVersion()) {
            log.info("  - Updated previous curve version for platform '%s' to '%s'".formatted(
                    platform.getPlatform().getPlatformName(),
                    platform.getPreviousCurveVersion()
            ));

            log.info("Updating scores for platform '%s'...".formatted(platform.getPlatform().getPlatformName()));
            Document finalDocument = document;
            EXECUTOR_SERVICE.execute(() -> {
                platform.updateLeaderboards();
                this.savePlatform(platform, finalDocument);
            }); // Update the leaderboards
        } else {
            this.savePlatform(platform, document);
        }
    }

    /**
     * Saves the platform.
     *
     * @param platform the platform to save
     * @param document the document to save
     */
    public void savePlatform(Platform platform, Document document) {
        document.put("currentCurveVersion", platform.getCurrentCurveVersion());
        MongoService.INSTANCE.getPlatformsCollection().replaceOne(Filters.eq("_id", platform.getPlatform().getPlatformName()), document);
    }

    /**
     * Gets the ScoreSaber platform.
     *
     * @return the ScoreSaber platform
     */
    public ScoreSaberPlatform getScoreSaberPlatform() {
        for (Platform platform : this.platforms) {
            if (platform.getPlatform().getPlatformName().equalsIgnoreCase(Platform.Platforms.SCORESABER.getPlatformName())) {
                return (ScoreSaberPlatform) platform;
            }
        }
        return null;
    }
}

